/*CXXR $Id: massdist.c 1348 2013-02-25 17:49:03Z arr $
 *CXXR
 *CXXR This file is part of CXXR, a project to refactor the R interpreter
 *CXXR into C++.  It may consist in whole or in part of program code and
 *CXXR documentation taken from the R project itself, incorporated into
 *CXXR CXXR (and possibly MODIFIED) under the terms of the GNU General Public
 *CXXR Licence.
 *CXXR 
 *CXXR CXXR is Copyright (C) 2008-13 Andrew R. Runnalls, subject to such other
 *CXXR copyrights and copyright restrictions as may be stated below.
 *CXXR 
 *CXXR CXXR is not part of the R project, and bugs and other issues should
 *CXXR not be reported via r-bugs or other R project channels; instead refer
 *CXXR to the CXXR website.
 *CXXR */

/*
 *  R : A Computer Language for Statistical Data Analysis
 *  Copyright (C) 1996-2004	Robert Gentleman and Ross Ihaka and the
 *				R Core Team
 *  Copyright (C) 2005		The R Foundation

 *  "HACKED" to allow weights by Adrian Baddeley
 *  Changes indicated by 'AB'
 * -------
 *  FIXME   Does he want 'COPYRIGHT' ?
 * -------
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, a copy is available at
 *  http://www.r-project.org/Licenses/
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <R_ext/Arith.h>
#include <R_ext/Applic.h>

void massdist(double *x,
	      double *xmass, /* AB: new variable */
	      int *nx,
	      double *xlow, double *xhigh,
	      double *y, int *ny)
{
    double fx, xdelta, xmi, xpos;   /* AB */
    int i, ix, ixmax, ixmin;

    ixmin = 0;
    ixmax = *ny - 2;
    /* AB: line deleted */
    xdelta = (*xhigh - *xlow) / (*ny - 1);

    for(i=0; i < *ny ; i++)
	y[i] = 0;

    for(i=0; i < *nx ; i++) {
	if(R_FINITE(x[i])) {
	    xpos = (x[i] - *xlow) / xdelta;
	    ix = floor(xpos);
	    fx = xpos - ix;
	    xmi = xmass[i];   /* AB: new line  */
	    if(ixmin <= ix && ix <= ixmax) {
		y[ix] += (1 - fx) * xmi;   /* AB */
		y[ix + 1] += fx * xmi; /* AB */
	    }
	    else if(ix == -1) {
		y[0] += fx * xmi;  /* AB */
	    }
	    else if(ix == ixmax + 1) {
		y[ix] += (1 - fx) * xmi;  /* AB */
	    }
	}
    }

    /* AB: lines deleted */
}
